# -*- encoding: utf-8 -*-
##############################################################################
#
#    Product loan module for OpenERP
#    Copyright (C) 2009-2011 Anevia S.A. (www.anevia.com),
#       code written by Raphaêl Valyi from Smile.fr,
#           now working at Akretion.com <rvalyi@akretion.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
import netsvc
import time
from datetime import datetime


class sale_order_line(osv.osv):
    _inherit = "sale.order.line"

    _columns = {
        'prodlot_id':fields.many2one('stock.production.lot', 'Serial number'),
        'is_loan': fields.related('order_id', 'is_loan', type='boolean', string='Is Loan?'),
    }

    def onload_is_loan(self, cr, uid, ids, order_is_loan, context=None):
        """hack used to set the is_loan hidden sale order line field to True if order is a loan. That hidden field is then used to set the prodlot_id field as required"""
        return {'value':{'is_loan':order_is_loan}}

    def loan_prodlot_change(self, cr, uid, ids, context=None):
        """Hack to make this module compatible with fleet_maintenance, without depending on it"""
        result = {}
        result['value'] = {}
        # The subfleet_required field is added by the module fleet_maintenance version >= 2011
        if self.pool.get('ir.model.fields').search(cr, uid, [('name', '=', 'subfleet_required'), ('model', '=', 'sale.order.line')])!=[]:
            result['value'].update({'subfleet_required': False})
        return result

    def _check_single_product(self, cr, uid, ids, context=None):
        for order_line in self.browse(cr, uid, ids):
            if order_line.is_loan and order_line.product_uom_qty != 1:
                return False
            return True

    _constraints = [
        (_check_single_product,
            """A loan can only have 1 product per loan line""", [])]

sale_order_line()


class sale_order(osv.osv):
    _inherit = "sale.order"

    def _default_name(self, cr, uid, context=None):
        if context is None:
            context = {}
        if context.get('is_loan', False):
            return self.pool.get('ir.sequence').get(cr, uid, 'loan')
        else:
            return self.pool.get('ir.sequence').get(cr, uid, 'sale.order')


    def _is_loan_expired(self, cr, uid, ids, field_name, arg, context=None):
        result = {}
        for order in self.browse(cr, uid, ids):
            result[order.id] = False
            if order.is_loan and order.loan_end_date < time.strftime('%Y-%m-%d'):
                for picking in order.picking_ids:
                    if (picking.type == 'in') and (picking.state in ('draft','assigned','confirmed')):
                        result[order.id] = True
        return result



    _defaults = {
        'name': _default_name
    }

    _columns = {
        'is_loan': fields.boolean('Is Loan?'),
        'is_loan_expired': fields.function(_is_loan_expired, method=True, type='boolean', string='Is Expired?'),
        'loan_start_date':fields.date('Loan Start Date', required=False),
        'loan_end_date':fields.date('Loan End Date', required=False),
    }


    # Set the name with the loan sequence instead of the
    # sale order sequence
    def copy(self, cr, uid, id, default=None, context={}):
        order_id = super(sale_order, self).copy(cr, uid, id, default, context)

        if context.get('is_loan', False):
            self.pool.get('sale.order').write(cr, uid, [order_id], {
                'name': self.pool.get('ir.sequence').get(cr, uid, 'loan')
            })

        return order_id


    def search(self, cr, uid, args, offset=0, limit=None, order=None,
             context=None, count=False):

        ids = super(sale_order, self).search(cr, uid, args, offset, limit,
                order, context=context, count=count)
        new_ids = []

        if context and context.has_key('loan_search') and context['loan_search'] is True:
            pos = 0
            is_loan_expired = False

            while pos < len(args):
                if args[pos][0] == 'is_loan_expired':
                    if args[pos][2] == "True":
                        is_loan_expired = True
                pos += 1

            if is_loan_expired:
                for loan_id in ids:
                    loan = self.pool.get('sale.order').browse(cr, uid, loan_id)
                    if loan.is_loan_expired:
                        new_ids.append(loan_id)
            else:
                new_ids = ids
        else:
                new_ids = ids

        return new_ids

    def action_ship_create(self, cr, uid, ids, *args):
        result = super(sale_order, self).action_ship_create(cr, uid, ids, *args)
        for order in self.browse(cr, uid, ids):
            if order.is_loan:
                loan_location_inside_id = order.shop_id.warehouse_id.property_stock_loan_inside_location.id
                loan_location_outside_id = order.shop_id.warehouse_id.property_stock_loan_outside_location.id
                #TODO strangely the following doesn't work, may be that's because of the ('res_id','=',res_id) test inside that method. 
                #self.pool.get('ir.property').get(cr, uid, 'property_stock_loan_inside_location', 'stock.warehouse')
                #print "action_ship_create action_ship_create action_ship_create"
                #print "loan_location_inside_id", loan_location_inside_id
                #print "loan_location_outside_id", loan_location_outside_id

                picking_id = False
                #remark: we consider there will be only one picking for the sale order now, this is a simplification
                for order_line in order.order_line:
                    for move in order_line.move_ids:
                        picking_id = move.picking_id.id

                if picking_id:
                    picking = self.pool.get('stock.picking').browse(cr, uid, picking_id)
                    for move in picking.move_lines:
                        #TODO copy the prodlot_id!!
                        self.pool.get('stock.move').write(cr, uid, move.id, {'date_planned': order.loan_start_date, \
                                                 'location_id': loan_location_inside_id, \
                                                 'location_dest_id': loan_location_outside_id, \
                                                 'prodlot_id':move.sale_line_id.prodlot_id.id })

                    self.pool.get('stock.picking').write(cr, uid, picking_id, {'min_date':order.loan_start_date, \
                                                                               'max_date':order.loan_start_date, \
                                                                               'invoice_state':'none'})
                    return_picking_id = self.pool.get('stock.picking').copy(cr, uid, picking_id)
                    #TODO alter a some fields!!!
                    return_picking = self.pool.get('stock.picking').browse(cr, uid, return_picking_id)
                    i = 0
                    for move in return_picking.move_lines:
                        self.pool.get('stock.move').write(cr, uid, move.id, {'date_planned': order.loan_end_date, \
                                                 'location_id': loan_location_outside_id, \
                                                 'location_dest_id': loan_location_inside_id, \
                                                 'sale_line_id': picking.move_lines[i].sale_line_id.id, \
                                                 'prodlot_id':picking.move_lines[i].sale_line_id.prodlot_id.id })
                        i += 1
                    self.pool.get('stock.picking').write(cr, uid, return_picking_id, {'type': 'in', 'min_date':order.loan_end_date, 'max_date':order.loan_end_date})
                    wf_service = netsvc.LocalService("workflow")
                    wf_service.trg_validate(uid, 'stock.picking', return_picking_id, 'test_assigned', cr)

        return result


    def _check_loan_date_order(self, cr, uid, ids, context=None):
        for order in self.browse(cr, uid, ids):
            if order.is_loan:
                date_start = datetime.strptime(order.loan_start_date, '%Y-%m-%d')
                date_end = datetime.strptime(order.loan_end_date, '%Y-%m-%d')
                if (date_start > date_end or date_end < datetime.today()):
                    return False
        return True

    def _check_product_quantity(self, cr, uid, ids, context=None):
        for order in self.browse(cr, uid, ids):
            for order_line in order.order_line:
                if order.is_loan and order_line.product_uom_qty != 1:
                    return False
        return True

    _constraints = [
        (_check_loan_date_order,
            """Start date should be before end date and should not be in the past !""", []),
        (_check_product_quantity,
            """A product loan carrying a serial number can only involve 1 product unit per serial number !""", [])
            ]

sale_order()
