# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherits from mrp.production for change it functionally"""

from osv import osv, fields
from tools.translate import _

class mrp_procurement(osv.osv):
    """inherits from mrp.production for change it functionally. Should allow do procurement based on lots not stock"""
    _inherit = "mrp.procurement"
    _order = 'date_planned desc'

    _columns = {
        'move_id': fields.one2many('stock.move', 'procurement_id', 'Reservations'),
        #dummy field to one2many realation procurements with mrp.procurements now the realition is procurement_id have many moves
        'dummy':fields.many2one('stock.move', 'Dummy')
        }

    def create(self, cr, uid, vals, context = {}):
        """overwrites this method to manage field move_id insertion"""
        move_id = False
        if 'move_id' in vals and type(vals['move_id']) != tuple:
            move_id = vals['move_id']
            del vals['move_id']
            
        procurement_id = super(mrp_procurement, self).create(cr, uid, vals, context = context)

        if move_id:
            if type(move_id) != list:
                move_ids = [move_id]
            else:
                move_ids = move_id
            #updates move with procurement_id
            for move_id in move_ids:
                self.pool.get('stock.move').write(cr, uid, move_id, {'procurement_id': procurement_id, })

        return procurement_id

    def action_confirm(self, cr, uid, ids, context={}):
        """overwrite all base method because now move_id is a one2many field not many2one"""
        for procurement in self.browse(cr, uid, ids):
            #CHECKME: Actualización desde tiny pero no se como funcionará con servicios, ya que siempre pide stock.
            if procurement.product_qty <= 0.00:
                raise osv.except_osv(_('Data Insufficient !'), _('Please check the Quantity of Procurement Order(s), it should not be less than 1!'))

            if procurement.product_id.type in ('product', 'consu'):
                if not procurement.move_id:
                    source = procurement.location_id.id
                    if procurement.procure_method == 'make_to_order':
                        source = procurement.product_id.product_tmpl_id.property_stock_procurement.id
                    self.pool.get('stock.move').create(cr, uid, {
                        'name': 'PROC:'+procurement.name,
                        'location_id': source,
                        'location_dest_id': procurement.location_id.id,
                        'product_id': procurement.product_id.id,
                        'product_qty':procurement.product_qty,
                        'product_uom': procurement.product_uom.id,
                        'date_planned': procurement.date_planned,
                        'state':'confirmed',
                        'procurement_id': procurement.id
                    })
                    self.write(cr, uid, [procurement.id], {'close_move':1})
                else:
                    if type(procurement.move_id) != list:
                        for obj_move_id in procurement.move_id:
                            if procurement.procure_method =='make_to_stock' and obj_move_id.state in ('waiting',):
                                self.pool.get('stock.move').write(cr, uid, [obj_move_id.id], {'state':'confirmed'})
        self.write(cr, uid, ids, {'state':'confirmed', 'message':''})
        return True

    def test_cancel(self, cr, uid, ids):
        """overwrites this method because nor move_id field is a collection"""
        for record in self.browse(cr, uid, ids):
            if record.move_id:
                for obj_move_id in record.move_id:
                    if obj_move_id.state == 'cancel':
                        return True
        return False

    def _check_make_to_stock_product(self, cr, uid, procurement, context={}):
        """overwrites this method because nor move_id field is a collection"""
        ok = True
        if procurement.move_id:
            for obj_move_id in procurement.move_id:
                if not (obj_move_id.state in ('done', 'assigned', 'cancel')):
                    ok = ok and self.pool.get('stock.move').action_assign(cr, uid, [obj_move_id.id])
                    cr.execute('select count(id) from stock_warehouse_orderpoint where product_id=%s', (procurement.product_id.id,))
                    if not cr.fetchone()[0]:
                        cr.execute('update mrp_procurement set message=%s where id=%s', (_('from stock and no minimum orderpoint rule defined'), procurement.id))
        return ok

    def check_all_procurement_done(self, cr, uid, ids, dummy = {}):
        """checks if all procurement moves are done and if production lot is assigned"""
        ok = True
        for procurement in self.browse(cr, uid, ids):
            if procurement.move_id:
                for obj_move_id in procurement.move_id:
                    if obj_move_id.state not in ['assigned', 'done']:
                        cr.execute('update mrp_procurement set message=%s,state=%s where id=%s', (_('Not assigned moves yet.'), 'exception', procurement.id))
                        ok = False
                    elif not obj_move_id.prodlot_id:
                        cr.execute('update mrp_procurement set message=%s,state=%s where id=%s', (_('Production lots required: Please set reservations\' production lots.'), 'exception', procurement.id))
                        ok = False
            else:
                ok = False
        return ok

    def check_move_done(self, cr, uid, ids, context={}):
        """overwrites this base method because move_id now is a one2many fiels not many2one"""
        res = True
        for proc in self.browse(cr, uid, ids, context):
            if proc.move_id:
                for obj_move_id in proc.move_id:
                    if not obj_move_id.state == 'done' or not obj_move_id.prodlot_id:
                        res = False
        return res

    def check_move_cancel(self, cr, uid, ids, context={}):
        """overwrites this base method because move_id now is a one2many fiels not many2one"""
        res = True
        ok = False
        for procurement in self.browse(cr, uid, ids, context):
            if procurement.move_id:
                ok = True
                for obj_move_id in procurement.move_id:
                    if not obj_move_id.state == 'cancel':
                        res = False
        return res and ok
        
mrp_procurement()