# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherit mrp.production, manages his methods allow creates a production without picking, beacause a liquid product not need procurement,
manage moves of production lots and final products"""

from osv import osv
from mx.DateTime import now
import time
from tools.translate import _

def compare( item_x, item_y ):
    """compare elements for all prodlots in location and child locations"""
    if item_x[2] < item_y[2]:
        rst = -1
    elif item_x[2] > item_y[2]:
        rst = 1
    else :
        rst = 0
    return rst

def compare_stock_move_objts( item_x, item_y ) :
    """compare objects with product quantity"""
    if item_x['product_qty'] < item_y['product_qty']:
        rst = 1
    elif item_x['product_qty'] > item_y['product_qty']:
        rst = -1
    else :
        rst = 0
    return rst

class mrp_production(osv.osv):
    """inherit mrp.production, manages his methods allow creates a production without picking, beacause a liquid product not need procurement,
    manage moves of production lots and final products"""
    _inherit = 'mrp.production'
    _order = 'date_planned desc'

    def write_relationship(self, cr, move_parent_id, move_child_id):
        """method to write to stock_move_history_ids"""
        if move_parent_id and move_child_id:
            if not isinstance(move_parent_id, list):
                move_parent_id = [move_parent_id]
            for parent_id in move_parent_id:
                if parent_id != move_child_id:
                    cr.execute("select parent_id from stock_move_history_ids where parent_id = %s and child_id = %s", (parent_id, move_child_id))
                    if not cr.rowcount:
                        cr.execute('insert into stock_move_history_ids (parent_id, child_id) values (%s,%s)', (parent_id, move_child_id))

        return True

    def recompute_with_used_prodlots(self, cr, uid, product_lines, production_id = False):
        """check the quantity of product for product_lines. Prepare the product_lines virtually for draw on prodlots with less quantity if exists"""
        for line in product_lines:
            if not production_id:
                production_id = line.production_id.id
            #gets the default prodlot and the location for it
            default_prodlot, default_prodlot_location, default_prodlot_qty = self.pool.get('stock.production.lot').get_default_production_lot(cr, uid, line.production_id.location_src_id.id, line.product_id.id, line.product_qty, True)
            composite_prodlots = []
            total_qty = line.product_qty
            #knows there is a lot that complete our needs
            res = self.pool.get('stock.production.lot').get_all_parent_location(cr, uid, line.production_id.location_src_id, line.product_id.id, 1, True)
            if default_prodlot:
                objects_toremove = []
                #if in the list found registies where the quantity is more that the line or is the default_prodlot, deletes this item
                for record in res:
                    if record[0] == default_prodlot and record[1] == default_prodlot_location and record[2] == default_prodlot_qty:
                        objects_toremove.append(record)
                        continue
                    if record[2] > line.product_qty:
                        objects_toremove.append(record)
                        continue
                if objects_toremove.append:
                    for record in objects_toremove:
                        res.remove(record)
                if res:
                    #order the list for quantity in element
                    res.sort(compare)
                    #In the ordered list, subtract the quantity of used prodlots to total quantity and add this prodlot in a list for then create a product line with his quantity
                    for record in res:
                        if total_qty >= record[2]:
                            composite_prodlots.append(record)
                            total_qty -= record[2]
                        else:
                            break
                if composite_prodlots:
                    #creates a product line for prodlot
                    for record in composite_prodlots:
                        self.pool.get('mrp.production.product.line').create(cr, uid, vals = {
                                                                            'name': line.name,
                                                                            'product_uom': line.product_uom.id,
                                                                            'product_qty': record[2],
                                                                            'production_id': line.production_id.id,
                                                                            'product_id': line.product_id.id,
                                                                    })
                #if there are quantity of product yet, update the original product_line with the new quantity else deletes teh product line
                if total_qty:
                    self.pool.get('mrp.production.product.line').write(cr, uid, line.id, {'product_qty': total_qty})
                else:
                    self.pool.get('mrp.production.product.line').unlink(cr, uid, [line.id])
            else:
                if res:
                    #orders the list for the quantities
                    res.sort(compare)
                    #In the ordered list, subtract the quantity of used prodlots to total quantity
                    #and add this prodlot in a list for then create a product line with his quantity
                    for record in res:
                        if total_qty >= record[2]:
                            composite_prodlots.append(record)
                            total_qty -= record[2]
                        elif not total_qty:
                            break
                        else:
                            composite_prodlots.append((record[0], record[1], total_qty))
                            total_qty = 0
                            break
                    if composite_prodlots:
                        #if consumed all the total_qty, can creates new product lines and deletes the old else not can create lines, because there isnt's product
                        if not total_qty:
                            for record in composite_prodlots:
                                self.pool.get('mrp.production.product.line').create(cr, uid, vals={
                                                                            'name': line.name,
                                                                            'product_uom': line.product_uom.id,
                                                                            'product_qty': record[2],
                                                                            'production_id': line.production_id.id,
                                                                            'product_id': line.product_id.id,
                                                                    })
                            self.pool.get('mrp.production.product.line').unlink(cr, uid, [line.id])
        return len(self.browse(cr, uid, production_id).product_lines)
    
    def check_if_picking(self, cr, uid, ids, dummy=[]):
        """method that checks if the products in the production are marked to do procurement"""
        for production in self.browse(cr, uid, ids):
            #goes around product lines:
            for line in production.product_lines:
                #if the product in this line not is liquid and marked that do procurement, there is create a picking else not and continue while not product lines end, in the final if not entered here return false, then no do the procurement
                if not line.product_id.liquid and line.product_id.do_procurement:
                    return True
            return False
        return True

    def action_compute(self, cr, uid, ids, dummy=[]):
        """overwrites this method to avoid recalculate product_lines if be marked to not procurement all product lines"""
        if not dummy:
            if self.check_if_picking(cr, uid, ids):
                return super(mrp_production, self).action_compute(cr, uid, ids)
            else:
                return False
        return super(mrp_production, self).action_compute(cr, uid, ids)

    def test_if_product(self, cr, uid, ids):
        """ovewrites this method to execute action-confirm with an option"""
        res = True
        for production in self.browse(cr, uid, ids):
            if not production.product_lines:
                bom_point = production.bom_id
                bom_id = production.bom_id.id
                if not bom_point:
                    bom_id = self.pool.get('mrp.bom')._bom_find(cr, uid, production.product_id.id, production.product_uom.id)
                if not bom_id:
                    return False
        return res


    def action_confirm(self, cr, uid, ids):
        """overwrites the base method, if the product is a liquid not do the procurement and production moves"""
        no_procurement_lines_data = []
        picking_id = False
        #goes around the ids of productions that objects
        for production in self.browse(cr, uid, ids):

            original_product_lines = production.product_lines

            #goes around the lines of products in the production
            for line in production.product_lines:
                #if the product is liquid or not want to do procurement
                if line.product_id.liquid or not line.product_id.do_procurement:
                    #insert the line in a memory list, for then recover it
                    no_procurement_lines_data.append(line)
                    #remove the line of production currently
                    self.pool.get('mrp.production.product.line').write(cr, uid, line.id, {'production_id': None})
                    #creates the rest of moves
            picking_id = super(mrp_production, self).action_confirm(cr, uid, ids)

            self.recompute_with_used_prodlots(cr, uid, original_product_lines, production_id = production.id)
            production = self.browse(cr, uid, [production.id])[0]
            
            for line in production.product_lines:
                #if the product is liquid
                if (line.product_id.liquid or not line.product_id.do_procurement) and not line in no_procurement_lines_data:
                    #insert the line in a memory list, for then recover it
                    no_procurement_lines_data.append(line)

            #set the product lot by default with less usability days and quatity
            notvalidprodlots = []
            for move in self.pool.get('stock.move').search(cr, uid, [('picking_id', '=', picking_id)], order="product_qty DESC"):
                obj_move = self.pool.get('stock.move').browse(cr, uid, move)
                #gets the default prodlot with the best conditions
                default_prodlot, prodlot_location, default_qty = self.pool.get('stock.production.lot').get_default_production_lot(cr, uid, obj_move.location_id.id, obj_move.product_id.id, obj_move.product_qty, True, notvalidprodlots)
                notvalidprodlots.append((default_prodlot, prodlot_location, default_qty))
                #updates the move with the product lot
                self.pool.get('stock.move').write(cr, uid, [obj_move.id], {'prodlot_id': default_prodlot, 'location_id': prodlot_location or obj_move.product_id.product_tmpl_id.property_stock_procurement.id})

            #now that the moves was created can create another time the lines with the liquids
            if len(no_procurement_lines_data)>0:
                notvalidprodlots = []
                #order the list of object by quantity desc
                no_procurement_lines_data.sort(compare_stock_move_objts)
                production_move_lines = []
                for procline_obj in no_procurement_lines_data:
                    self.pool.get('mrp.production.product.line').write(cr, uid, procline_obj.id, {'production_id': procline_obj.production_id.id})
                    default_prodlot, prodlot_location, default_qty = self.pool.get('stock.production.lot').get_default_production_lot(cr, uid, procline_obj.product_id.product_tmpl_id.property_stock_procurement.id,
                    procline_obj.product_id.id, procline_obj.product_qty, True, notvalidprodlots)
                    
                    notvalidprodlots.append((default_prodlot, prodlot_location, default_qty))
                    #creates the move in stock_move for the liquids products and product not procure
                    production_move = self.pool.get('stock.move').create(cr, uid, vals={
                                                      'product_uom' : procline_obj.product_uom.id,
                                                      'product_uos_qty' : procline_obj.product_uos_qty,
                                                      'date' : now().strftime('%Y-%m-%d %H:%M:%S'),
                                                      'product_qty' : procline_obj.product_qty,
                                                      'product_uos' : procline_obj.product_uos,
                                                      'location_id' : prodlot_location or procline_obj.product_id.product_tmpl_id.property_stock_procurement.id,
                                                      'product_id' : procline_obj.product_id.id,
                                                      'prodlot_id' : default_prodlot,
                                                      'name' : 'NoProc:' + str(procline_obj.product_id.product_tmpl_id.property_stock_procurement.id) + str(no_procurement_lines_data.index(procline_obj)) + 'TO' + now().strftime('%Y-%m-%d %H:%M:%S'),
                                                      'date_planned' : now().strftime('%Y-%m-%d %H:%M:%S'),
                                                      'state' : 'confirmed',
                                                      'location_dest_id' : procline_obj.product_id.product_tmpl_id.property_stock_production.id,
                                                  })
                                                  
                    #searches the final move in production and insert a registry in move_history_ids and other in mrp_production_move_ids
                    id_move_product_toproduct = self.pool.get('stock.move').search(cr, uid, [('production_id', '=', production.id)])
                    if id_move_product_toproduct:
                        for move_product_id in id_move_product_toproduct:
                            #writes move relationship
                            self.write_relationship(cr, production_move, move_product_id)
                        production_move_lines.append(production_move)

                #move_lines de produccion: ", (production.move_lines)
                #move_lines a añadir a producción: ", (production_move_lines)

                for obj_move_id in production.move_lines:
                    production_move_lines.append(obj_move_id.id)
                #move_lines a añadir a producción al final: ", (production_move_lines)
                
                #production_move_lines.extend(production.move_lines)
                self.write(cr, uid, [production.id], {'move_lines': [(6, 0, production_move_lines)]})
                #cr.execute('insert into mrp_production_move_ids (production_id, move_id) values (%s,%s)', (production.id, production_move))
        return picking_id

    # pylint: disable-msg=W0141
    def action_no_picking(self, cr, uid, ids):
        """creates the final move for the production and a move by product in the bom from
        product's procurement location or a child her to production location"""
        for production in self.browse(cr, uid, ids):
            original_product_lines = map(lambda x: x.id, production.product_lines)
            self.pool.get('mrp.production.product.line').write(cr, uid, map(lambda x: x.id, production.product_lines), {'production_id': None})
            production = self.browse(cr, uid, production.id)
            #create empty picking, delete
            super(mrp_production, self).action_confirm(cr, uid, ids)
            self.pool.get('mrp.production.product.line').write(cr, uid, original_product_lines, {'production_id': production.id})
            production = self.browse(cr, uid, production.id)
            
            #final move
            final_moves_ids = self.pool.get('stock.move').search(cr, uid, [('production_id', '=', production.id)])

            res_final_id = final_moves_ids[0]

            if production.product_lines:
                notvalidprodlots = []
                order_product_lines = self.pool.get('mrp.production.product.line').search(cr, uid, [('production_id', '=', production.id)], order="product_qty DESC")
                for line in order_product_lines:
                    obj_line = self.pool.get('mrp.production.product.line').browse(cr, uid, line)
                    
                    #search the default prodlot and the location of this prodlot
                    default_prodlot, prodlot_location, default_qty = self.pool.get('stock.production.lot').get_default_production_lot(cr, uid, obj_line.product_id.product_tmpl_id.property_stock_procurement.id,
                    obj_line.product_id.id, obj_line.product_qty, True, notvalidprodlots)

                    notvalidprodlots.append((default_prodlot, prodlot_location, default_qty))
                    if not prodlot_location:
                        prodlot_location = obj_line.product_id.product_tmpl_id.property_stock_procurement.id
                    #creates the move in stock_move for the product not procure from deafult location of prodlot to production location
                    production_move = self.pool.get('stock.move').create(cr, uid, vals={
                                                      'product_uom' : obj_line.product_uom.id,
                                                      'product_uos_qty' : obj_line.product_uos_qty,
                                                      'date' : now().strftime('%Y-%m-%d %H:%M:%S'),
                                                      'product_qty' : obj_line.product_qty,
                                                      'product_uos' : obj_line.product_uos,
                                                      'location_id' : prodlot_location,
                                                      'product_id' : obj_line.product_id.id,
                                                      'prodlot_id' : default_prodlot,
                                                      'name' : 'NoProc:' + str(obj_line.product_id.product_tmpl_id.property_stock_procurement.id) + str(production.product_lines.index(obj_line)) + 'TO' + now().strftime('%Y-%m-%d %H:%M:%S'),
                                                      'date_planned' : now().strftime('%Y-%m-%d %H:%M:%S'),
                                                      'state' : 'draft',
                                                      'move_dest_id': res_final_id,
                                                      'location_dest_id' : obj_line.product_id.product_tmpl_id.property_stock_production.id,
                                                  })
                    #searches the final move in production and insert a registry in mrp_production_move_ids
                    if res_final_id:
                        cr.execute('insert into mrp_production_move_ids (production_id, move_id) values (%s,%s)', (production.id, production_move))

            self.write(cr, uid, [production.id], {'state':'confirmed'})
        return True

    def action_ready(self, cr, uid, ids):
        """check if exists final_moves, if not exists is create"""
        for production in self.browse(cr, uid, ids):
            if not production.move_created_ids:
                data = {
                    'name':'PROD:'+production.name,
                    'date_planned': production.date_planned,
                    'product_id': production.product_id.id,
                    'product_qty': production.product_qty,
                    'product_uom': production.product_uom.id,
                    'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                    'product_uos': production.product_uos and production.product_uos.id or False,
                    'location_id': production.product_id.product_tmpl_id.property_stock_production.id,
                    'location_dest_id': production.location_dest_id.id,
                    'move_dest_id': production.move_prod_id.id,
                    'state': 'waiting'
                }
                res_final_id = self.pool.get('stock.move').create(cr, uid, data)

                self.write(cr, uid, [production.id], {'move_created_ids': [(6, 0, [res_final_id])]})


        return super(mrp_production, self).action_ready(cr, uid, ids)

    def action_in_production(self, cr, uid, ids):
        """overwrites the method because need set a production lot to the products that not have"""
        for production in self.browse(cr, uid, ids):
            #gets the production move for this production
            cr.execute('select move_id from mrp_production_move_ids where production_id = %s', (production.id,))
            res = cr.fetchall()[:]
            #goes around the moves
            for production_move in res:
                obj_production_move = self.pool.get('stock.move').browse(cr, uid, production_move[0])
                #if not have production lot
                if not obj_production_move.prodlot_id:
                    #gets the default production lot for this product in this location
                    default_prodlot = self.pool.get('stock.production.lot').get_default_production_lot(cr, uid, obj_production_move.location_id.id, obj_production_move.product_id.id, obj_production_move.product_qty, False)
                    if default_prodlot:
                        #updates the production lot of the product
                        self.pool.get('stock.move').write(cr, uid, obj_production_move.id, {'prodlot_id': default_prodlot, 'state': 'assigned'})
                    else:
                        #if there isn't default prodlot raises an exception indicates how to
                        raise osv.except_osv(_('Error!'),
                            _('The production not can continue because not be found a valid production lot in the location "%s" for the product "%s", set a production lot or choose other location for this product.') % (
                            obj_production_move.location_id.name, obj_production_move.product_id.name))
                #checks if the move has parent if not create it in the history moves
                cr.execute('SELECT child_id FROM stock_move_history_ids WHERE child_id=%s', (obj_production_move.id,))
                if not cr.rowcount:
                    if obj_production_move.prodlot_id.id:
                        parent_move_id = self.pool.get('stock.move').get_move_in_location(cr, uid, obj_production_move.location_id.id, obj_production_move.prodlot_id.id, obj_production_move.product_qty)
                    else:
                        parent_move_id = self.pool.get('stock.move').get_move_in_location(cr, uid, obj_production_move.location_id.id, default_prodlot, obj_production_move.product_qty)
                    if parent_move_id:
                        self.write_relationship(cr, parent_move_id, obj_production_move.id)
                        
            return super(mrp_production, self).action_in_production(cr, uid, ids)
            
    def action_production_end(self, cr, uid, ids, dummy=[]):
        """overwrites the base method, remove the history moves creation"""
        move_ids = []
        final_moves = []
        for production in self.browse(cr, uid, ids):
            for res in production.move_lines:
                move_ids.append(res.id)
            new_moves = [x.id for x in production.move_created_ids]
            final_moves.extend(new_moves)
            self.pool.get('stock.move').write(cr, uid, move_ids, {'move_dest_id': new_moves[0]})
            vals = {'state':'confirmed'}
            
            self.pool.get('stock.move').write(cr, uid, new_moves, vals)
            if not production.date_finnished:
                self.write(cr, uid, [production.id],
                        {'date_finnished': time.strftime('%Y-%m-%d %H:%M:%S')})
            self.pool.get('stock.move').check_assign(cr, uid, new_moves)
            self.pool.get('stock.move').action_done(cr, uid, new_moves)

            for production_move_id in new_moves:
                for move_obj_id in production.move_lines:
                    self.write_relationship(cr, move_obj_id.id, production_move_id)

            #It is private method but I need to use
            # pylint: disable-msg=E1101
            self._costs_generate(cr, uid, production)

        self.pool.get('stock.move').action_done(cr, uid, move_ids)
        self.write(cr,  uid, ids, {'state': 'done'})

        #set real finish hour to production final moves
        self.pool.get('stock.move').write(cr, uid, final_moves, {'date_planned': time.strftime('%Y-%m-%d %H:%M:%S')})
        return True
    
    def test_location(self, cr, uid, ids, dummy=[]):
        """method fire from mrp_workflow when click on confirm production, checks if can confirm the production"""
        valid = True
        for production in self.browse(cr, uid, ids):
            #if the destination only acepts liquids
            if production.location_dest_id.liquid_location:
                #checks if the product not is liquid
                if not production.product_id.liquid:
                    valid = False
                    raise osv.except_osv(_('Error'), _("Only assign miscible products to this location"))
                #checks if the product not is the same type that the product in destination if exists
                res = self.pool.get('stock.real.report.prodlots').search(cr, uid, [('location_id', '=', production.location_dest_id.id)])
                if len(res) == 1:
                    obj_real_report_prodlot_id = self.pool.get('stock.real.report.prodlots').browse(cr, uid, res[0])
                    for move in production.move_lines:
                        if move.product_uom and move.prodlot_id and move.location_dest_id:
                            #obtains the base uom of move uom
                            base_uom_id = self.pool.get('product.uom').search(cr, uid, [('category_id', '=', move.product_uom.category_id.id), ('factor', '=', 1.0)])
                            #converts the qunatity to base uom
                            move.product_qty = self.pool.get('product.uom')._compute_qty(cr, uid, move.product_uom.id, move.product_qty, base_uom_id[0])
                            #if all stock in dest location  is consumed in production the location is valid because after confirm production the location was empty
                            if move.prodlot_id.id == obj_real_report_prodlot_id.prodlot_id.id and move.product_qty == obj_real_report_prodlot_id.name and move.location_id.id == production.location_dest_id.id:
                                return True
                        valid = False
                        raise osv.except_osv(_('Error'), _("Can not mix products that are not of the same type in this mixtures destination"))
                elif len(res) > 1:
                    raise osv.except_osv(_('Error'), _("Invalid mixtures location. The location has more that one product"))
            else:
                if production.product_id.liquid and 'internal' in production.location_dest_id.usage:
                    raise osv.except_osv(_('Error'), _("Cannot assign miscible products to this location"))
        return valid
    
    def product_id_change(self, cr, uid, ids, product):
        """overwrites the event fires when changed the product_id, completed the source location
        with the corresponding property"""
        if not product:
            return {}
        res = super(mrp_production, self).product_id_change(cr, uid, ids, product)
        obj_product_id = self.pool.get('product.product').browse(cr, uid, product)
        res['value']['location_src_id'] = obj_product_id.product_tmpl_id.property_raw.id
        return {'value':res['value']}

    def write(self, cr, uid, ids, vals, context = {}):
        """overwrites this method for managed the changes in the move_lines"""
        #if not is a list, converts it to list
        if type(ids) != type([]):
            ids = [ids]
        #variable to return
        res = 0
        #variable that will content the new line if exists
        line = 0
        #if is updating the move lines
        if ('move_lines' in vals) and (vals['move_lines']):
            for production_id in ids:
                obj_production_id = self.browse(cr, uid, production_id)
                #variable that content the old move lines in the bbdd, before update
                move_lines_bbdd = obj_production_id.move_lines
                #update
                res = super(mrp_production, self).write(cr, uid, production_id, vals, context = context)

                #gets an object with the changes
                obj_production_id = self.browse(cr, uid, production_id)
                #search the new moves
                #goes around the moves now in bbdd
                for line_bbdd in obj_production_id.move_lines:
                    #goes around the moves old in bbdd
                    for line_bbdd_old in move_lines_bbdd:
                        if line_bbdd.id == line_bbdd_old.id:
                            line = False
                            break
                        line = True
                    #if encountered a new move
                    if line:
                        line = False
                        #gets the move final, the move with the product final
                        production_move_ids = self.pool.get('stock.move').search(cr, uid, [('production_id', '=', obj_production_id.id)])
                        line_added = line_bbdd.id
                        #if encountered the production move, inserts the parental relationships
                        if production_move_ids:
                            for production_move_id in production_move_ids:
                                self.write_relationship(cr, line_added, production_move_id)
            return res
        return super(mrp_production, self).write(cr, uid, ids, vals, context = context)

mrp_production()     