# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""add functionally for differentiate liquid product and not liquid product"""

from osv import fields, osv
from tools.translate import _
from stock_production_lot_sequence import get_default_seq_id_for_prodlot

class product_product(osv.osv):
    """add functionally for differentiate liquid product and not liquid product"""
    _inherit = "product.product"
    _columns = {
                'liquid': fields.boolean('Miscible Product', help="If the product can be mixed check this, for not create a procurement in a production order, and can put inside mix locations and only these."),
                'do_procurement': fields.boolean('Do procurement in production', help="If not check, in production, if not is a liquid product won't do a procurement move."),
                'sequence_id':fields.many2one('ir.sequence', 'Prodlots Serial'),
                'default_code' : fields.char('Code', size=64, required=True)
                }
    _defaults = {
                 'liquid': lambda *a: 0,
                 'do_procurement': lambda *a: 1,
                 'sequence_id': lambda x, y, z, c: x.pool.get('ir.sequence').search(y, z, [('code', '=', 'stock.lot.serial')])[0] or False,
                 }

    _sql_constraints = [
        ('default_code_uniq', 'unique (default_code)', _('The product default code must be unique !')),
    ]
    
    def check_if_procurement(self, cr, uid, ids, vals):
        """checks if the product has configurate well the procurement"""
        if 'liquid' in vals and vals['liquid'] == True:
            return True
        if 'do_procurement' in vals and vals['do_procurement'] == True and len(ids) > 0:
            if type(ids) != type([]):
                ids = [ids]
            for product_id in ids:
                obj_product_id = self.browse(cr, uid, product_id)
                if obj_product_id and obj_product_id.liquid and (not 'liquid' in vals or vals['liquid'] == True):
                    raise osv.except_osv(_('Error!'),
                        _('Cannnot put this product to do procurement, because this poduct is marked that miscible and the miscible products not do procurement.'))
        return False

    def write(self, cr, uid, ids, vals, context={}):
        """overwrites write method for checks if the procurement for the product is configurate well"""
        change = self.check_if_procurement(cr, uid, ids, vals)
        if change:
            vals['do_procurement'] = 0
        return super(product_product, self).write(cr, uid, ids, vals, context=context)

    def create(self, cr, uid, vals, context={}):
        """overwrites create method for checks if the procurement for the product is configurate well and creates sequence to production lots"""
        change = self.check_if_procurement(cr, uid, [], vals)
        if change:
            vals['do_procurement'] = 0
            
        product_id = super(product_product, self).create(cr, uid, vals, context=context)

        #Manages the sequence number

        sequence_id = get_default_seq_id_for_prodlot(cr, uid, product_id)
        self.write(cr, uid, product_id, {'sequence_id': sequence_id})

        return product_id

    
product_product()

class product_template(osv.osv):
    """add functionally for differentiate liquid product and not liquid product"""
    _inherit = "product.template"
    _columns = {
        'property_raw': fields.property(
        'stock.location',
        type='many2one',
        relation='stock.location',
        string="Raw Materials Location",
        method=True,
        view_load=True,
        help="For the current product (template), this stock location will be used, instead of the default one, as the source where search the products needs for a production of this product (template)."),
	    'property_mix': fields.property(
	    'stock.location',
	    type='many2one',
	    relation='stock.location',
	    string='Mix Moves Location',
	    method=True,
	    view_load=True,
	    help="Location where will go the products that form a mix"), 
            }

product_template()