# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherits from purchase.order, creates a method that in purchase workflow check the products and locations"""

from osv import osv, fields
from lxml import etree
from tools.translate import _

class purchase_order(osv.osv):
    """inherits from purchase.order, creates a method that in purchase workflow check the products and locations"""
    _inherit = 'purchase.order'

    def test_location(self, cr, uid, ids, dummy=[]):
        """method fire from purchase_workflow when click on confirm purchase order, checks if can confirm the purchase"""
        valid = True
        if type(ids) != type([]):
            ids = [ids]
        for purchase in self.browse(cr, uid, ids):
            #checks if the product not is liquid
            for line in purchase.order_line:
                #if liquid location
                if line.location_id.liquid_location:
                    res = self.pool.get('stock.real.report.prodlots').search(cr, uid, [('location_id', '=', line.location_id.id)])
                    #checks if the product not is the same type that the product in destination if exists
                    if len(res) == 1:
                        obj_real_report_prodlot_id = self.pool.get('stock.real.report.prodlots').browse(cr, uid, res[0])
                        if obj_real_report_prodlot_id.product_id.id != line.product_id.id:
                            valid = False
                            raise osv.except_osv(_('Error'), _("Can not mix products that are not of the same type in this mixtures destination."))
                    elif len(res) > 1:
                        raise osv.except_osv(_('Error'), _("Invalid mixtures location. The location has more that one product"))
                else:
                    if line.product_id.liquid and 'internal' in line.location_id.usage:
                        raise osv.except_osv(_('Error'), _("Cannot assign miscible products to this location"))
        return valid
    
purchase_order()

class purchase_order_line(osv.osv):
    """inherits from purchase.order.line adds functional fields"""
    _inherit = 'purchase.order.line'

    _columns = {
            'location_id': fields.many2one('stock.location', 'Destination', required=True),
            }

    _defaults = {
            'location_id': lambda self, cr, uid, context: context.get('location_id', False)
            }

    def _check_location(self, cr, uid, ids):
        """checks if the location only acepts liquid and if the product is liquid"""
        for obj_order_line_id in self.browse(cr, uid, ids):
            if obj_order_line_id.location_id and obj_order_line_id.location_id.liquid_location and (not obj_order_line_id.product_id.liquid):
                return False
            elif obj_order_line_id.location_id and not obj_order_line_id.location_id.liquid_location and  obj_order_line_id.product_id.liquid and 'internal' in obj_order_line_id.location_id.usage:
                return False
        return True

#    def _check_if_service(self, cr, uid, ids):
#        """checks if the product is a service, if service cannot have location in order line"""
#        for obj_order_line_id in self.browse(cr, uid, ids):
#            if obj_order_line_id.product_id.product_tmpl_id and obj_order_line_id.product_id.product_tmpl_id.type != 'product' and obj_order_line_id.location_id:
#                return False
#        return True
#
#    def _check_if_product(self, cr, uid, ids):
#        """checks if the product is a product type, if product type require to have location in order line"""
#        for obj_order_line_id in self.browse(cr, uid, ids):
#            if obj_order_line_id.product_id.product_tmpl_id and obj_order_line_id.product_id.product_tmpl_id.type == 'product' and not obj_order_line_id.location_id:
#                return False
#        return True

    _constraints = [
    (_check_location, 'Destination not valid for this product', ['location_id']),
#    (_check_if_service,
#            'This product is a service or consumable, cannot move to a destination. Empty destination.',
#            ['product_id']),
#    (_check_if_product,
#            'This product required destination.',
#            ['product_id'])
    ]

purchase_order_line()