# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherits from inventory object, beacuse want to allow move products with prodlot, now not used prodlot to move"""

from osv import fields, osv
from tools.translate import _
import time

class stock_inventory(osv.osv):
    """checks prodlots in inventory moves"""
    _inherit = 'stock.inventory'

    def action_done(self, cr, uid, ids, context=None):
        """overwrites this method because the inventory base not insert the production lot in the moves and move products without production lot"""
        if context is None:
            context = {}

        #quantity of product is the quantity in location not quantity in location + quantity in child location
        context['compute_child'] = False

        for inventory in self.browse(cr, uid, ids):
            move_ids = []
            #goes around the inventory lines
            for line in inventory.inventory_line_id:
                parent_moves = []
                #gets and calculate the stock that must have in the move
                cr.execute('select name from stock_report_prodlots where location_id = %s and product_id = %s and prodlot_id = %s', (line.location_id.id, line.product_id.id, line.prodlot_id.id))
                res_quantities = cr.fetchall()
                if res_quantities:
                    quantity = res_quantities[len(res_quantities)-1][0]
                    quantity = quantity * line.product_uom.factor
                    final_qty = quantity - line.product_qty
                else:
                   final_qty = line.product_qty

                if final_qty and round(final_qty, 2) != 0.00:
                    location_id = line.product_id.product_tmpl_id.property_stock_inventory.id
                    value = {
                        'name': line.name or 'INV:' + str(line.inventory_id.id) + ':' + line.inventory_id.name,
                        'product_id': line.product_id.id,
                        'product_uom': line.product_uom.id,
                        'date': inventory.date,
                        'date_planned': inventory.date,
                        'state': 'assigned',
                        'prodlot_id': line.prodlot_id.id
                    }
                    if final_qty > 0:
                        value.update( {
                            'product_qty': final_qty,
                            'location_dest_id': location_id,
                            'location_id': line.location_id.id,
                        })
                    else:
                        value.update( {
                            'product_qty': -final_qty,
                            'location_dest_id': line.location_id.id,
                            'location_id': location_id,
                        })

                    move_id = self.pool.get('stock.move').create(cr, uid, value)
                    move_obj = self.pool.get('stock.move').browse(cr, uid, move_id)
                    move_ids.append(move_id)

                    #gets the parent move for the inventory move
                    parent_moves = self.pool.get('stock.move').get_move_in_location(cr, uid, line.location_id.id, line.prodlot_id.id, line.product_qty)
                    if parent_moves and not isinstance(parent_moves, list):
                        parent_moves = [parent_moves]

                        for parent_move in parent_moves:
                            self.pool.get('stock.move').write_relationship(cr, parent_move, move_obj.id)

            if len(move_ids):
                self.pool.get('stock.move').action_done(cr, uid, move_ids, context=context)
            self.write(cr, uid, [inventory.id], {'state': 'done', 'date_done': time.strftime('%Y-%m-%d %H:%M:%S'), 'move_ids': [(6, 0, move_ids)]})

        return True

    def unlink(self, cr, uid, ids, context={}):
        """overwrites this method to not allow delete inventory on state done"""
        for inventory_id in self.browse(cr, uid, ids):
            if inventory_id.state == 'done':
                raise osv.except_osv(_("Error"), _("Can't delete done inventory."))

        return super(stock_inventory, self).unlink(cr, uid, ids, context=context)

stock_inventory()

class stock_inventory_line(osv.osv):
    """adds prodlot_id to inventory moves and checks it"""
    _inherit = 'stock.inventory.line'

    _columns = {
               'prodlot_id' : fields.many2one('stock.production.lot', 'Production Lot', required=True, help="Production lot of the product about that perform the inventory"),
               'name': fields.char('Description', size=64)
            }

    def onchange_lot_id(self, cr, uid, ids, prodlot_id=False, context=None):
        """overwrites this event for change the name by product code + prodlot name"""
        if prodlot_id:
            prodlot_obj_id = self.pool.get('stock.production.lot').browse(cr, uid, prodlot_id)
            return {'value': {'name': prodlot_obj_id.product_id.product_tmpl_id.name + " (" + prodlot_obj_id.name + ")"}}
        return {}

    def on_change_product_id(self, cr, uid, ids, location_id=False, product=False, uom=False, context = {}):
        """overwrites this method adding the functionally if the product is liquid shows his production lot"""
        result = super(stock_inventory_line, self).on_change_product_id(cr, uid, ids, location_id, product, uom, context)
        if result and location_id:
            if self.pool.get('product.product').browse(cr, uid, product).liquid and self.pool.get('stock.location').browse(cr, uid, location_id).liquid_location:
                default_prodlot = self.pool.get('stock.production.lot').get_default_production_lot(cr, uid, location_id, product, result['value']['product_qty'], False)
                if default_prodlot:
                    result['value']['prodlot_id'] = default_prodlot
        return result

    def on_change_location_id(self, cr, uid, ids, location_id=False, product_id=False, prodlot_id=False, product_qty=False):
        """on changes loaction_id field search for producton lot else not found return None"""
        result = {}
        if location_id and product_id and product_qty and self.pool.get('product.product').browse(cr, uid, product_id).liquid and self.pool.get('stock.location').browse(cr, uid, location_id).liquid_location:
            default_prodlot = self.pool.get('stock.production.lot').get_default_production_lot(cr, uid, location_id, product_id, product_qty, False)
            if default_prodlot:
                result = {'value': {'prodlot_id': default_prodlot}}
        else:
            result = {'value': {'prodlot_id': None}}
        return result
    
stock_inventory_line()