# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherits stock.production lot, adds methods that get production lots hierarchical or manages production lot sequence"""

from osv import fields, osv
from tools.translate import _
import time

class stock_production_lot(osv.osv):
    """inherits stock.production lot, adds methods that get production lots hierarchical or manages production lot sequence"""
    _inherit = 'stock.production.lot'
    _order = "date desc"

    def _get_supplier(self, cr, uid, ids, field_name, arg, context=None):
        """gets the name of the supplier"""
        if context == None: context = {}
        res = {}
        for prodlot_id in ids:
            res[prodlot_id] = None
            move_ids = self.pool.get('stock.move').search(cr, uid, [('prodlot_id', '=', prodlot_id)])
            for move_id in move_ids:
                move = self.pool.get('stock.move').browse(cr, uid, move_id)
                if move.location_id.name == 'Suppliers':
                    if move.picking_id:
                        res[prodlot_id] = move.picking_id.address_id.partner_id.name
                        break
        return res

    def make_sscc(self, cr, uid, context={}):
        """return production lot number"""
        if 'product_id' in context and context['product_id']:
            sequence_obj_id = self.pool.get('product.product').browse(cr, uid, context['product_id']).sequence_id
            if sequence_obj_id:
                sequence_id = sequence_obj_id.id
                sequence_number = self.pool.get('ir.sequence').get_id(cr, uid, sequence_id)
                return sequence_number

        sequence = self.pool.get('ir.sequence').get(cr, uid, 'stock.lot.serial')
        return sequence

    _columns = {
                'is_mix': fields.boolean('Mix'),
                'supplier': fields.function(_get_supplier, method=True, string="Supplier", type='char')
            }
    _defaults = {'is_mix': lambda *a: 0,
                 'name': make_sscc}

    def get_all_parent_location(self, cr, uid, obj_location, product_id, qty, deep):
        """searches all the prodlots, quantities and locations for a location and his child locations"""
        locations_list = []
        if deep:
            locations = self.pool.get('stock.location').search(cr, uid, [('location_id', 'child_of', [obj_location.id])])
        else:
            locations = [obj_location.id]
        #gets the prodlot, the quantity of this prodlot and the location where is the prodlot by location, product and quantity
        cr.execute('select prodlot_id, location_id, name as id from stock_report_prodlots where location_id in (%s) and product_id = %s and name >= %s' % (",".join(map(str, locations)), product_id, qty))
        if cr.rowcount > 0:
            #extend the list with data of the query
            locations_list.extend(cr.fetchall()[:])
        return locations_list
    
    def get_default_production_lot(self, cr, uid, location, product_id, qty_needed, deep, notvalidprodlots = []):
        """gets a prodution lot for a product in location, gets the production lot with less usability time and with less quatity"""
        #gets all prodlots in the location with this product and stock are enough
        obj_location_id = self.pool.get('stock.location').browse(cr, uid, location)
        default_prodlot = False
        prodlot_location = False
        default_qty = False
        res = self.get_all_parent_location(cr, uid, obj_location_id, product_id, qty_needed, deep)
        #by default the first registry
        if res:
            #checks if the prodlots not are used, if used remove it
            if notvalidprodlots:
                recordstoremove = []
                for record in res:
                    for prodlot in notvalidprodlots:
                        if record == prodlot:
                            recordstoremove.append(record)
                            break
                for toremove in recordstoremove:
                    res.remove(toremove)
            if res:
                firstobj_production_lot = self.browse(cr, uid, res[0][0])
                prodlot_location = res[0][1]
                default_qty = res[0][2]
                expirydate = firstobj_production_lot.dlc
                default_prodlot = firstobj_production_lot.id
                for record in res:
                    obj_prodlot = self.browse(cr, uid, record[0])
                    if obj_prodlot.dlc:
                        if expirydate:
                            #if dlc of this product is less that dlc for default, changes the default values
                            if time.strptime(expirydate, '%Y-%m-%d %H:%M:%S') > time.strptime(obj_prodlot.dlc, '%Y-%m-%d %H:%M:%S'):
                                expirydate = obj_prodlot.dlc
                                default_qty = record[2]
                                default_prodlot = obj_prodlot.id
                                prodlot_location = record[1]
                        else:
                            #if the first prodlot not have expirydate, changes the default values
                            expirydate = obj_prodlot.dlc
                            default_prodlot = obj_prodlot.id
                            prodlot_location = record[1]
                            default_qty = record[2]
                    if record[2] < default_qty:
                        default_qty = record[2]
                        expirydate = obj_prodlot.dlc
                        default_prodlot = obj_prodlot.id
                        prodlot_location = record[1]
        if deep:
            return default_prodlot, prodlot_location , default_qty
        else:
            return default_prodlot

    def unlink(self, cr, uid, ids, context = {}):
        """Overwrites unlink method to avoid delete lots with moves"""
        for prodlot_id in self.browse(cr, uid, ids):
            move_ids = self.pool.get('stock.move').search(cr, uid, [('prodlot_id', '=', prodlot_id.id)])
            if move_ids:
                raise osv.except_osv(_('Error !'), _('You cannot delete this lot because it has moves depends on!'))

        return super(stock_production_lot, self).unlink(cr, uid, ids, context = context)
    
stock_production_lot()