# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""wizard that replace completely stock traceability wizard of stock for only show the first time to a prodlot is parent
in the traceability"""

import wizard
import pooler
from tools.translate import _

def action_traceability(tree_type='move_history_ids', field='tracking_id'):
    """call to recursive method that shows tree view"""
    def open_tab(self, cr, uid, data, context):
        """recursive method that searches tree view and prodlots ids"""
        obj = pooler.get_pool(cr.dbname).get('stock.move')
        ids = obj.search(cr, uid, [(field, 'in', data['ids'])])

        if not isinstance(ids, list):
            ids = [ids]

        if ids:
            # pylint: disable-msg=W0141
            ids = ','.join(map(str, ids))

            #filter ids: not can have parents or production final moves
            cr.execute("""select stock_move.id from stock_move inner join stock_production_lot
            on stock_move.prodlot_id = stock_production_lot.id where stock_move.id in (%s) and
            (stock_move.id not in (select child_id from stock_move_history_ids) or
            (is_mix = True and prodlot_id not in (select distinct prodlot_id from stock_move AS sm
            inner join stock_move_history_ids on sm.id = parent_id
            where child_id in (stock_move.id))) or production_id is not null)""" % (ids))

            ids = []
            for (parent_id,) in cr.fetchall():
                ids.append(parent_id)

        cr.execute('select id from ir_ui_view where model=%s and field_parent=%s and type=%s', ('valid.stock.moves', tree_type, 'tree'))
        view_id = cr.fetchone()[0]
        # pylint: disable-msg=W0141
        value = {
            'domain': "[('id','in',["+','.join(map(str, ids))+"])]",
            'name': ((tree_type=='move_history_ids') and _('Upstream Traceability')) or _('Downstream Traceability'),
            'view_type': 'tree',
            'res_model': 'valid.stock.moves',
            'field_parent': tree_type,
            'view_id': (view_id,'View'),
            'type': 'ir.actions.act_window'
        }
        return value
    return open_tab

class wiz_journal(wizard.interface):
    """down traceability for trac lots"""
    states = {
        'init': {
            'actions': [],
            'result': {'type': 'action', 'action': action_traceability('move_history_ids2'), 'state':'end'}
        }
    }
wiz_journal('stock.liquid.traceability.tracking.downstream')

class wiz_journal2(wizard.interface):
    """up traceability for trac lots"""
    states = {
        'init': {
            'actions': [],
            'result': {'type': 'action', 'action': action_traceability(), 'state':'end'}
        }
    }
wiz_journal2('stock.liquid.traceability.tracking.upstream')

class wiz_journal3(wizard.interface):
    """up traceability for prodlots"""
    states = {
        'init': {
            'actions': [],
            'result': {'type': 'action', 'action': action_traceability(field='prodlot_id'), 'state':'end'}
        }
    }
wiz_journal3('stock.liquid.traceability.lot.upstream')

class wiz_journal4(wizard.interface):
    """down traceability for prodlots"""
    states = {
        'init': {
            'actions': [],
            'result': {'type': 'action', 'action': action_traceability('move_history_ids2', 'prodlot_id'), 'state':'end'}
        }
    }
wiz_journal4('stock.liquid.traceability.lot.downstream')


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: