# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""fill an inventory object with inventory line with all products in the specific location"""

import wizard
import pooler
from tools.translate import _

INVENTORY_FORM = """<?xml version = "1.0"?>
<form string = "Fill Inventory">
    <separator colspan = "4" string = "Fill Inventory for specific location" />
    <field name = "location_id"/>
    <newline/>
    <field name = "recursive"/>
    <newline/>

</form>
"""

INVENTORY_FIELDS = {
    'location_id' : {
        'string':'Location',
        'type':'many2one',
        'relation':'stock.location',
        'required':True
    },
    'recursive' : {'string':'Include all childs for the location', 'type':'boolean'}
}



def _fill_inventory(self, cr, uid, data, context):
    """fill an inventory object with inventory line with all products in the specific location"""
    #gets the objects
    pool = pooler.get_pool(cr.dbname)
    inventory_line_obj = pooler.get_pool(cr.dbname).get('stock.inventory.line')
    location_obj = pooler.get_pool(cr.dbname).get('stock.location')
    res = {}
    res_location = {}
    #if recursive check
    # pylint: disable-msg=W0212
    if data['form']['recursive'] :
        location_ids = location_obj.search(cr, uid, [('location_id', 'child_of', [data['form']['location_id']])])
	    #gets the child locations products
        for location in location_ids :
            res = location_obj._product_get(cr, uid, location)
            res_location[location] = res
    else:
        context.update({'compute_child':False})
    res = location_obj._product_get(cr, uid, data['form']['location_id'], context = context)
    res_location[data['form']['location_id']] = res

    product_ids = []
    #goes around the locations
    for location in res_location.keys():
        res = res_location[location]
	    #goes around the products
        for product_id in res.keys():
            #product_ids.append(product_id)
            cr.execute('select prodlot_id, name as id from stock_report_prodlots where location_id = %s and product_id = %s and name > 0', (location, product_id))
            prodlots = cr.fetchall()
            for prodlot_id in prodlots:
                prod = pool.get('product.product').browse(cr, uid, [product_id])[0]
                uom = prod.uom_id.id
                context.update({'uom': uom})
                amount = prodlot_id[1] * prod.uom_id.factor
                prodlot = prodlot_id[0]

                if (amount and prod.type == 'product'):
                    line_ids = inventory_line_obj.search(cr, uid, [('inventory_id', '=', data['id']), ('location_id', '=', location),
                    ('product_id', '=', product_id), ('product_uom', '=', uom), ('product_qty', '=', amount), ('prodlot_id', '=', prodlot)])
                    if not len(line_ids):
                        inventory_line = {'inventory_id':data['id'], 'location_id':location, 'product_id':product_id, 'product_uom':uom, 'product_qty':amount, 'prodlot_id': prodlot}
                        inventory_line_obj.create(cr, uid, inventory_line)
                    product_ids.append(product_id)

    if(len(product_ids)==0):
        raise wizard.except_wizard(_('Message !'), _('No product in this location.'))
    return {}



class fill_inventory(wizard.interface):
    """fill an inventory object with inventory line with all products in the specific location"""
    states = {
        'init' : {
            'actions' : [],
            'result' : {'type' : 'form',
                    'arch' : INVENTORY_FORM,
                    'fields' : INVENTORY_FIELDS,
                    'state' : [('end', 'Cancel'), ('fill_inventory', 'Fill Inventory') ]}
        },
        'fill_inventory' : {
            'actions' : [],
            'result' : {'type' : 'action', 'action': _fill_inventory, 'state' : 'end'}
        },
    }
fill_inventory("stock.liquid.traceability.fill_inventory")



