# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherits stock_location to improve the computation of product_qty"""

from osv import fields, osv

class stock_location(osv.osv):
    """inherits stock_location to improve the computation of product_qty"""
    _inherit="stock.location"

    def _product_qty_available(self, cr, uid, ids, field_names, arg, context={}):
        res = {}
        for id in ids:
            res[id] = {}.fromkeys(field_names, 0.0)
            if 'stock_uom' in field_names:
                res[id]['stock_uom'] = False

        for loc in self.browse(cr, uid, ids):
            context['location'] = [loc.id]
            context['compute_child'] = context.get('compute_child', True)
            if context.get('product_id', False):
                prod = self.pool.get('product.product').browse(cr, uid, context['product_id'], context)

                if 'stock_real' in field_names:
                    res[loc.id]['stock_real'] = prod.qty_available
                if 'stock_virtual' in field_names:
                    res[loc.id]['stock_virtual'] = prod.virtual_available
                if 'stock_uom' in field_names:
                    res[loc.id]['stock_uom'] = (prod.uom_id.id, prod.uom_id.name)

        return res

    _columns = {
        'stock_real': fields.function(_product_qty_available, method=True, type='float', string='Real Stock', multi="stock"),
        'stock_virtual': fields.function(_product_qty_available, method=True, type='float', string='Virtual Stock', multi="stock"),
        'stock_uom': fields.function(_product_qty_available, method=True, type='many2one', relation="product.uom", string="UoM", multi="stock")
    }

stock_location()