# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherits from stock.production.lot adds functionally for warn prodlot expiration"""

from osv import osv, fields
from datetime import datetime, timedelta
from tools.translate import _
import pooler

def _get_date(dtype):
    """overwirtes argument function for defaults methods"""
    def calc_date(self, cr, uid, context=None):
        """overwrites this method from product_expiry because if not is organic product and not have use_time, completed it with today date"""
        if context is None: context = {}
        if context.get('product_id', False):
            product = pooler.get_pool(cr.dbname).get('product.product').browse(cr, uid, [context['product_id']])[0]
            duree = getattr(product, dtype) or 0
            if duree:
                date = datetime.today() + timedelta(days=duree)
                return date.strftime('%Y-%m-%d %H:%M:%S')

        return False
    return calc_date

class stock_production_lot(osv.osv):
    """inherits from stock.production.lot adds functionally for warn prodlot expiration"""
    _inherit = 'stock.production.lot'

    def _get_if_expired(self, cr, uid, ids, field_name, arg, context=None):
        """get if prodlots is expired based on removal_date"""
        if context is None: context = {}
        res = {}
        for obj_prodlot_id in self.browse(cr, uid, ids):
            if obj_prodlot_id.removal_date:
                value = False
                #check if prodlot is expired
                if obj_prodlot_id.removal_date < datetime.now().strftime('%Y-%m-%d'):
                    value = True
                res[obj_prodlot_id.id] = value
            else:
                res[obj_prodlot_id.id] = False
        return res

    _columns = {
        'expired': fields.function(_get_if_expired, method=True, type="boolean", string="Expired",
            store={'stock.production.lot': (lambda self, cr, uid, ids, c={}: ids, None, 20)}),
    }

    _defaults = {
        'dlc': _get_date('life_time'),
        'dluo': _get_date('use_time'),
        'removal_date': _get_date('removal_time'),
        'alert_date': _get_date('alert_time'),
    }

    #pylint: disable-msg=W0613
    def searchfor_expired_prodlots(self, cr, uid, automatic=False, use_new_cursor=False, context={}):
        """cron that search for expired prodlots and mark its with expired everyday, send a notification too"""
        today = datetime.now().strftime('%Y-%m-%d')

        ids = self.search(cr, uid, [('removal_date', '<', today), ('expired', '=', False)])
        if ids:
            self.write(cr, uid, ids, {})

            group_id = self.pool.get('res.groups').search(cr, uid, [('name', '=', 'Production Lots / Expiration Notifications')])

            if group_id:
                group_id = group_id[0]

                cr.execute("select uid from res_groups_users_rel where gid = %s" % (group_id,))

                res = cr.fetchall()

                if not isinstance(ids, list):
                    ids = [ids]
                    
                not_stock_prodlots = []

                for prodlot_id in ids:
                    cr.execute("select * from stock_real_report_prodlots inner join stock_location \
                    on stock_real_report_prodlots.location_id = stock_location.id where prodlot_id = %s and usage = 'internal'" % (prodlot_id,))
                    if not cr.rowcount:
                        not_stock_prodlots.append(prodlot_id)

                ids = list(set(ids) - set(not_stock_prodlots))

                #get a string coma list from object case prodlots collection
                # pylint: disable-msg=W0141
                expired_lots_names = ','.join(map(str, map(lambda x:x.name, self.browse(cr, uid, ids))))

                #
                # unicode parse if necessary
                #
                expired_lots_names = (isinstance(expired_lots_names, str) and unicode(expired_lots_names, 'utf-8', errors='replace')) or expired_lots_names

                message = _("New production lots expired. Now not can you use this prodlots definitely.\n\nLots names: %s\n\n") % (expired_lots_names,)

                for (user_id,) in res:
                    self.pool.get('res.request').create(cr, uid, {
                            'name': _("Production Lots Expired"),
                            'body': message,
                            'state': 'waiting',
                            'act_from': uid,
                            'act_to': user_id,
                            'priority': '0'
                        })

        return True

stock_production_lot()