# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""inherits stock.production lot, adds methods that get production lots hierarchical or manages production lot sequence"""

from osv import fields, osv
from tools import config

class stock_production_lot(osv.osv):
    """inherits stock.production lot, adds unit price"""
    _inherit = 'stock.production.lot'

    def _get_stock(self, cr, uid, ids, field_name, arg, context={}):
        if 'location_id' not in context:
            locations = self.pool.get('stock.location').search(cr, uid, [('usage', '=', 'internal')], context=context)
        else:
            locations = context['location_id'] and [context['location_id']] or []

        if isinstance(ids, (int, long)):
            ids = [ids]

        res = {}.fromkeys(ids, 0.0)

        if locations:
            cr.execute('''select
                    prodlot_id,
                    sum(name)
                from
                    stock_report_prodlots
                where
                    location_id in ('''+','.join(map(str, locations))+''')  and
                    prodlot_id in  ('''+','.join(map(str, ids))+''')
                group by
                    prodlot_id
            ''')
            result = cr.fetchall()
            for prodlot_id, stock in result:
                res[prodlot_id] = stock * self.browse(cr, uid, prodlot_id).product_uom.factor
        return res

    def _stock_search(self, cr, uid, obj, name, args, context):
        locations = self.pool.get('stock.location').search(cr, uid, [('usage', '=', 'internal')])
        cr.execute('''select
                prodlot_id,
                sum(name)
            from
                stock_report_prodlots
            where
                location_id in ('''+','.join(map(str, locations)) + ''')
            group by
                prodlot_id
            having  sum(name)  ''' + str(args[0][1]) + ''' ''' + str(args[0][2])
        )
        res = cr.fetchall()
        ids = [('id', 'in', map(lambda x: x[0], res))]
        return ids

    _columns = {
                'unit_price': fields.float('Cost Price', required=True, digits=(16, int(config['price_accuracy']))),
                'product_uom': fields.many2one('product.uom', 'Product UOM', required=True),
                'stock_available': fields.function(_get_stock, fnct_search=_stock_search, method=True, type="float", string="Available", select="2"),
            }

    _defaults = {
        'unit_price': lambda *a: 0.00
    }

    def on_change_product_id(self, cr, uid, ids, product):
        """on change product id in production lot form fills unit_price and product_uom"""
        if not product:
            return {}
        product_obj_id = self.pool.get('product.product').browse(cr, uid, product)
        result = {'unit_price': product_obj_id.product_tmpl_id.standard_price, 'product_uom': product_obj_id.product_tmpl_id.uom_id.id}
        return {'value': result}

stock_production_lot()